﻿/* Bomb Class for Kaboom
 * 
 * This class represents bombs dropped by the computer.
 * The bomber moves randomly from side to side and drops bombs.  
 * The player must catch these in the buckets they control.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Kaboom
{
    class Bomb : Microsoft.Xna.Framework.DrawableGameComponent
    {
        // bomb  properties
        protected Texture2D texture;  // bomb image
        protected Rectangle spriteRectangle;  // bomb area
        protected Vector2 position;  // bomb position
        protected int Yspeed;  // bomb speed on the y-axis
        protected int Xspeed;  // bomb speed on the x-axis
        protected Random random; // used for randomizing the bombs start position
        protected static int levelSpeed=7;  // level speed factor

        // Width and height of sprite in texture
        protected const int BOMBWIDTH = 40;
        protected const int BOMBHEIGHT = 40;

        // Bomb constructor
        public Bomb(Game game, ref Texture2D theTexture, Bomber computer)
            : base(game)
        {

            texture = theTexture;
            position = new Vector2();

            // create the source rectangle
            // This represents where the sprite picture is in the surface
            
            spriteRectangle = new Rectangle(0, 0, BOMBWIDTH, BOMBHEIGHT);
            
            // initialize the random number generator and put the bomb in its start position
            
            random = new Random(this.GetHashCode());
            
            PutinStartPosition(computer.xCoordinate());

        }
        /// <summary>
        /// Initialize Bomb position and velocity
        /// </summary>
        protected void PutinStartPosition(float bomberXposition)
        {
            position.X = bomberXposition+20;
            position.Y = 40;
            Yspeed = levelSpeed;
            Xspeed = random.Next(3) - 1;
        }

        /// <summary>
        /// Allows the game component to draw your content in the game screen
        /// </summary>
        public override void Draw(GameTime gameTime)
        {
            // Get the current sprite batch
            SpriteBatch sBatch = (SpriteBatch)Game.Services.GetService(typeof(SpriteBatch));

            // Draw the Bomb
            sBatch.Draw(texture, position, spriteRectangle, Color.White);

            base.Draw(gameTime);
        }

        /// <summary>
        /// Initialization of the bomb
        /// </summary>
        public override void Initialize()
        {            
            levelSpeed = 7;
            Yspeed = 7;
            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            

            // Move BOMB
            position.Y += Yspeed;
           // position.X += Xspeed;

            base.Update(gameTime);
        }

        /// <summary>
        /// Check if the bombs intersects with the specified rectangle
        /// </summary>
        /// <param name="rect">test rectangle</param>
        /// <returns> true, if has a collision</returns>
        public bool CheckCollision(Rectangle rect)
        {
            Rectangle spriterect = new Rectangle((int)position.X, (int)position.Y,
              BOMBWIDTH, BOMBHEIGHT);
            return spriterect.Intersects(rect);
        }

        public static void nextLevel() {

            levelSpeed+=2;
        }
        public static void reset()
        {

            levelSpeed = 7;
        }
        
    }
}
