﻿/* Bucket Class for Kaboom
 * 
 * This is the class for the player object in the game.
 * The player starts with 3 buckets to catch bombs.  If the player misses a bomb,
 * then they lose a bucket.  When they lose all their buckets the game is over.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Kaboom
{
    class Bucket : Microsoft.Xna.Framework.DrawableGameComponent
    {
        // bucket properties
        protected Texture2D texture;  // bucket image
        protected Rectangle spriteRectangle;  // bucket size
        protected Vector2 position; // bucket position
        protected int numberOfBuckets;  // number of buckets

        // width and height of sprite in texture
        protected const int BUCKETWIDTH = 159;
        protected const int BUCKETHEIGHT = 35;

        // screen area
        protected Rectangle screenBounds;

        // Bucket constructor
        public Bucket(Game game, ref Texture2D theTexture) : base(game) {
            texture = theTexture;
            position = new Vector2();
            numberOfBuckets = 3;
            // create the source rectangle
            // this represents where the sprite picture is in the surface
            spriteRectangle = new Rectangle(0, 0, BUCKETWIDTH, BUCKETHEIGHT);
/*#if XBOX360
      // on the 360, we need to be careful about the TV's "safe" area.
      //screenBounds = new Rectangle(
      //  (int)(Game.Window.ClientBounds.Width * 0.03f),
      //  (int)(Game.Window.ClientBounds.Height * 0.03f),
      //  Game.Window.ClientBounds.Width -
      //  (int)(Game.Window.ClientBounds.Width * 0.03f),
      //  Game.Window.ClientBounds.Height -
      //  (int)(Game.Window.ClientBounds.Height * 0.03f));
//#else
            screenBounds = new Rectangle(-10 , 0,
              800,
              600);
#endif*/

            screenBounds = new Rectangle(-10, 0,
              800,
              600);
        
        }
        /// <summary>
        /// Put the Bucket in your start position in the game
        /// </summary>
        public void PutinStartPosition()
        {
            numberOfBuckets = 3;
            position.X = 400;
            position.Y = 580 - BUCKETHEIGHT;
        }


        /// <summary>
        /// Update the bucket
        /// </summary>
        /// <param name="gameTime">provides a snapshopt of timing values</param>
        public override void Update(GameTime gameTime)
        {
            // Move the bucket with the Xbox controller
            GamePadState gamepadstatus = GamePad.GetState(PlayerIndex.One);
            if (gamepadstatus.ThumbSticks.Left.X > 0) {
               position.X += 10;

            }
            else if (gamepadstatus.ThumbSticks.Left.X < 0) {
                position.X -= 10;
            
            }
            
            // move the bucket with the keyboard
            KeyboardState keyboard = Keyboard.GetState();

            if (keyboard.IsKeyDown(Keys.Left))
            {
                position.X -= 10;
            }
            if (keyboard.IsKeyDown(Keys.Right))
            {
                position.X += 10;
            }

            // Keep the bucket inside the screen
            if (position.X < screenBounds.Left)
            {
                position.X = screenBounds.Left;
            }
            if (position.X > screenBounds.Width - BUCKETWIDTH)
            {
                position.X = screenBounds.Width - BUCKETWIDTH;
            }
            base.Update(gameTime);
        }

        /// <summary>
        /// Draw the Bucket sprite
        /// </summary>
        public override void Draw(GameTime gameTime)
        {
            // get the current sprite batch
            SpriteBatch sBatch =
              (SpriteBatch)Game.Services.GetService(typeof(SpriteBatch));

            // draw the bucket
            Vector2 temppos = position;
            for (int i = 1; i <= numberOfBuckets; i++)
            {
                sBatch.Draw(texture, temppos, spriteRectangle, Color.White);
                temppos.Y -= BUCKETHEIGHT;
            }
            base.Draw(gameTime);
        }

        /// <summary>
        /// Get the bound rectangle of the bucket position in screen
        /// </summary>
        public Rectangle GetBounds()
        {
            return new Rectangle((int)position.X, ((int)position.Y-BUCKETHEIGHT*numberOfBuckets),
              BUCKETWIDTH, BUCKETHEIGHT * numberOfBuckets);
        }
        public bool decreaseBucketNumber()
        {
           
            numberOfBuckets--;
            if (numberOfBuckets == 0) return true;
            return false;
        }
        public void bucketNumberReset()
        {

            numberOfBuckets = 3;
            
        }
        public void goToNextLevel() {

            position.X = 800 / 2;
            position.Y = 600 - BUCKETHEIGHT;
        }
    }
}
