/* Aladdin Hussein
 * Robert A. Muir
 * 
 * CIS 297
 * Winter 2010 - Evan Musu
 * 
 * Assignment 4, Final Game Project: Kaboom
 * Language: C#, XNA framework
 * Type: 2D Game
 * Platform: XBox360 
 * 
A mad bomber is on the loose and he is tossing explosives at you!  
You must catch his lit bombs in a bucket of water to douse the fuse

The players opponent, the mad bomber, is positioned at the top of a wall near the top
of the game screen.  
The bomber moves from side to side and drops his explosives at random intervals.  
The player controls up to three buckets at the bottom of the screen.  
The player must move these containers side to side in a position to catch the oncoming
bombs. 

Every bomb the player catches is kept track of in the score.
If the player misses a bomb, they lose a bucket.  When they lose all three buckets,
the game is over.

 * Known Bugs/Issues:  Right now there is no game difficulty cap.  As difficulty increases,
 * the bomber's movement speed increases.  This may cause a problem with determining the
 * collision of the bomber at the edge of the screen.  It ends up going to fast and getting
 * stuck on the edges.  We think we can prevent this from happening by having a difficulty
 * cap, which will cap the bomber speed.  In order to find a logical fix, it would require adjusting the
 * bomber, the player speed, and the bomb drop speed for game tuning.  We did not have time
 * for this.  Our current fix involves resetting the bomber's positions at the beginning of every level.
 * This ensures that he does not stay stuck.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Kaboom
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // games textures
        private Texture2D backgroundTexture; // game background image
        private Texture2D bombTexture; // bomb sprite texture
        private Texture2D bomberTexture;  // computer bomber sprite texture
        private Texture2D bucketTexture;  // player bucket sprite texture
        private Texture2D gameOverTexture; // game over image
        private Texture2D introTexture; // game intro image
        private Texture2D helpTexture;  // help screen image

        // Game stuff
        private Bucket player;  // bucket object the player controls
        private Bomber computer;  // the bomber who drops the bombs
        private KeyboardState keyboard; // previous keyboard state
        private KeyboardState cK; // current keyboard state
        private SpriteFont gameFont; // fonts for in game text
        private SpriteFont msgsFont;
        private SpriteFont largeFont;
        private GamePadState gamepadstatus;  // current gamepad state
        private GamePadState oldgamepadstatus;  // previous gamepad state
        
        private const int STARTBOMBCOUNT = 0; // the initial number of bombs
        private const int ADDBOMBTIME = 500; // amount of time before next bomb is dropped
        
        private int lastTickCount;  // game time measurement
        private int bombCount;  // count of the number of bombs, used for determining level partitioning
        private int totalScore;  // total number of bombs the player has caught
        private int gameLevel;  //  current difficulty level of the game (1..n), higher level means higher difficulty

        //Audio Stuff
        private AudioEngine audioEngine;
        private WaveBank waveBank;
        private SoundBank soundBank;
        private Cue backgroundSound;


        Video video;//The actual video
        VideoPlayer vplayer;//The player for the video
        Texture2D videoTexture;//The texture to be drawn for the video
        
        // game status
        enum GameState { Start, InGame, GameOver,NextLeve,Pause,Quite,HelpScreen };
        GameState currentGameState = GameState.Start;


        public Game1()  // default constructor
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            graphics.PreferredBackBufferHeight = 600;
            graphics.PreferredBackBufferWidth =800;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {            
            base.Initialize();
            Window.Title = "Kaboom!";
            backgroundSound = soundBank.GetCue("backgroundTracklow");
            backgroundSound.Play();
            gameLevel = 1;
             
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Add the SpriteBatch service
            Services.AddService(typeof(SpriteBatch), spriteBatch);

            // load all textures
            backgroundTexture = Content.Load<Texture2D>("Background");
            bombTexture = Content.Load<Texture2D>("bomb");
            bomberTexture = Content.Load<Texture2D>("bomber");
            gameOverTexture = Content.Load<Texture2D>("gameover");
            bucketTexture = Content.Load<Texture2D>("bucket");
            introTexture = Content.Load<Texture2D>("introTexture");
            helpTexture = Content.Load<Texture2D>("helpTexture");

            // load game font
            gameFont = Content.Load<SpriteFont>("font");

            msgsFont = Content.Load<SpriteFont>("msgsfont");
            largeFont = Content.Load<SpriteFont>("largeFont");

            // load audio
            audioEngine = new AudioEngine(@"Content\audio\gameSounds.xgs");
            waveBank = new WaveBank(audioEngine, @"Content\audio\Wave Bank.xwb");
            soundBank = new SoundBank(audioEngine, @"Content\audio\Sound Bank.xsb");

            // load video
            video = Content.Load<Video>("explosionv");
            vplayer = new VideoPlayer();//Creates the video player
         
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
            // nothing to unload right now, everything managed by garabage collection.
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
             gamepadstatus = GamePad.GetState(PlayerIndex.One);

            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back ==
              ButtonState.Pressed)
                this.Exit();

            cK = Keyboard.GetState();

            // determine what to do based on current gameState and input
            switch (currentGameState)
            {
                case GameState.Start:
                // manage start state
                    if (cK.IsKeyDown(Keys.Space) && keyboard.IsKeyUp(Keys.Space) || gamepadstatus.Buttons.A == ButtonState.Pressed && oldgamepadstatus.Buttons.A == ButtonState.Released)
                    {
                        currentGameState = GameState.InGame;
                                          
                            Start();
                       
                    }
                    if (cK.IsKeyDown(Keys.H) && keyboard.IsKeyUp(Keys.H) ||gamepadstatus.Buttons.Y == ButtonState.Pressed && oldgamepadstatus.Buttons.Y == ButtonState.Released)
                    {
                        currentGameState = GameState.HelpScreen;
                        
                        Start();

                    }
                    if (cK.IsKeyDown(Keys.Escape) && keyboard.IsKeyUp(Keys.Escape))
                    {
                        Exit();

                    }
                    break;

                case GameState.InGame:   
                // manage  core game play
                            // Run the game
                            DoGameLogic();

                            if (cK.IsKeyDown(Keys.Enter) && keyboard.IsKeyUp(Keys.Enter)|| gamepadstatus.Buttons.A == ButtonState.Pressed && oldgamepadstatus.Buttons.A == ButtonState.Released)
                            {
                                currentGameState = GameState.Pause;
                            }

                            if (cK.IsKeyDown(Keys.Escape) && keyboard.IsKeyUp(Keys.Escape))
                            {
                                Exit();
                            }
                   break;
                case GameState.GameOver:
                // manage game over state

                   if (cK.IsKeyDown(Keys.Space) && keyboard.IsKeyUp(Keys.Space) || gamepadstatus.Buttons.A == ButtonState.Pressed && oldgamepadstatus.Buttons.A == ButtonState.Released)
                    {
                       
                        RemoveAllBombs();
                        lastTickCount = System.Environment.TickCount;
                        computer.reset();
                        Bomb.reset();
                        vplayer.Stop();
                        videoTexture = null;
                        currentGameState = GameState.Start;
                        Start();
                    }
                    if (cK.IsKeyDown(Keys.Escape) && keyboard.IsKeyUp(Keys.Escape))
                    {
                        Exit();

                    }
                    break;

                case GameState.NextLeve:
                // manage advancement of game to next difficulty level
                    if (cK.IsKeyDown(Keys.Space) && keyboard.IsKeyUp(Keys.Space) || gamepadstatus.Buttons.A == ButtonState.Pressed && oldgamepadstatus.Buttons.A == ButtonState.Released)
                    {
                        currentGameState = GameState.InGame;
                        
                        computer.goTonextLevel();
                        Bomb.nextLevel();
                        player.goToNextLevel();
                    }
                    if (cK.IsKeyDown(Keys.Escape) && keyboard.IsKeyUp(Keys.Escape))
                    {
                        Exit();

                    }
                    break;


                case GameState.HelpScreen:
                // manage help screen 
                    if (cK.IsKeyDown(Keys.Space) && keyboard.IsKeyUp(Keys.Space) || gamepadstatus.Buttons.A == ButtonState.Pressed && oldgamepadstatus.Buttons.A == ButtonState.Released)
                    {
                        currentGameState = GameState.Start;
                        
                      
                    }
                    if (cK.IsKeyDown(Keys.Escape) && keyboard.IsKeyUp(Keys.Escape))
                    {
                        Exit();

                    }
                    break;

                case GameState.Pause:
                // manage player game pauses
                    if (cK.IsKeyDown(Keys.Enter) && keyboard.IsKeyUp(Keys.Enter) || gamepadstatus.Buttons.A == ButtonState.Pressed && oldgamepadstatus.Buttons.A == ButtonState.Released)
                    {
                        currentGameState = GameState.InGame;

                    }
                    if(gamepadstatus.Buttons.Y == ButtonState.Pressed && oldgamepadstatus.Buttons.Y == ButtonState.Released){
                    
                    currentGameState = GameState.Start;
                    }
                    break;
                
            }

            // advance input sequence
            // previous input state gets current input state, for next update
            keyboard = cK; 
            oldgamepadstatus = gamepadstatus;
           
            // Update all other components
            base.Update(gameTime);
        }

    

        /// <summary>
        /// Initialize the game round
        /// </summary>
        private void Start()
        { // Create (if necessary) and put the player in start position
            if (player == null)
            {
                // Add the player component
                player = new Bucket(this, ref bucketTexture);
                Components.Add(player);
            }
            if (computer == null)
            {
                // Add the computer bomber 
                computer = new Bomber(this, ref bomberTexture);
                Components.Add(computer);
            }


            player.PutinStartPosition();
            computer.PutinStartPosition();
            // Add the Bombes
           
                Components.Add(new Bomb(this, ref bombTexture,computer));



               
            // initialize counters
            lastTickCount = System.Environment.TickCount;
            bombCount = STARTBOMBCOUNT;

         
            bombCount=0;
            totalScore=0;
            gameLevel=1;

        }

        /// <summary>
        /// Manages the core game play.
        /// </summary>
        private void DoGameLogic()
        {
            // Check collisions
            bool hasCollision = false;
            Rectangle bucketRectangle = player.GetBounds();

            foreach (GameComponent gc in Components)
            {
                if (gc is Bomb)
                {
                    hasCollision = ((Bomb)gc).CheckCollision(bucketRectangle);
                    {
                        if (hasCollision)
                        {
                           

                                // this represents a player successfully catching a bomb
                                soundBank.PlayCue("splash");
                                bombCount++;

                                Components.Remove(gc);
                                if (bombCount == 20) {
                                    gameLevel++;
                                    currentGameState = GameState.NextLeve;
                                totalScore = totalScore + bombCount;
                                bombCount = 0;
                                    RemoveAllBombs();
                               
                                }

                            break;
                        }
                    }
                    hasCollision=((Bomb)gc).CheckCollision(new Rectangle(0,590,800,10));
                    if(hasCollision)
                    {
                        // this represents the player missing a bomb
                        soundBank.PlayCue("drop");

                        if (player.decreaseBucketNumber()) {
                            
                            RemoveAllBombs();
                            currentGameState = GameState.GameOver;
                            backgroundSound.Pause();

                            vplayer.Play(video);
                            soundBank.PlayCue("explosion");

                        }
                    Components.Remove(gc);
                    break;
                    }
                }
            }
            CheckforNewBomb();
        }

        /// <summary>
        /// Remove all bombs from the game
        /// </summary>
        private void RemoveAllBombs()
        {
            for (int i = 0; i < Components.Count; i++)
            {
                if (Components[i] is Bomb)
                {
                    Components.RemoveAt(i);
                    i--;
                }
            }
        }

        /// <summary>
        /// Check if is a moment for a new bomb!
        /// </summary>
        private void CheckforNewBomb()
        {
            // Add a bomb each ADDBOMBTIME
            if ((System.Environment.TickCount - lastTickCount) > ADDBOMBTIME)
            {
                lastTickCount = System.Environment.TickCount;
                Components.Add(new Bomb(this, ref bombTexture,computer));
                
                soundBank.PlayCue("fuse");
            
             
                
            }
        }
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            // Draw background texture in a separate pass
            spriteBatch.Begin();

            //            spriteBatch.Draw(backgroundTexture, new Rectangle(0, 0,
            //graphics.GraphicsDevice.DisplayMode.Width, graphics.GraphicsDevice.DisplayMode.Height),
            //              Color.LightGray);
            
            spriteBatch.Draw(backgroundTexture, new Rectangle(0, 0,800, 600),Color.LightGray);

            if (currentGameState == GameState.Start)
            {
                spriteBatch.Draw(introTexture, new Rectangle(0, 0, 800, 600), Color.LightGray);
                spriteBatch.DrawString(msgsFont, "Press space / A to start,\nH / Y for help", new Vector2(100, 300), Color.White);
            }
            if (currentGameState == GameState.GameOver) {
                spriteBatch.Draw(gameOverTexture, new Rectangle(0, 0, 800, 600), Color.LightGray);
            }

            if (currentGameState == GameState.NextLeve) {
                spriteBatch.DrawString(largeFont, "Level " + gameLevel  , new Vector2(200, 300), Color.Yellow);
                spriteBatch.DrawString(msgsFont,  "Press space / A to continue " , new Vector2(200, 350), Color.White);
            }
            if (currentGameState == GameState.Pause)
            {
              spriteBatch.DrawString(largeFont, "Paused", new Vector2(100, 300), Color.Yellow); 
                spriteBatch.DrawString(msgsFont, "Press Y on Xbox Control", new Vector2(100, 350), Color.White);
                spriteBatch.DrawString(msgsFont, "To go to Main Menu", new Vector2(100, 400), Color.White);
            }
            if (currentGameState == GameState.HelpScreen)
            {
                spriteBatch.Draw(helpTexture, new Rectangle(0, 0, 800, 600), Color.LightGray);
            }
            spriteBatch.End();


            if (currentGameState == GameState.InGame)
            {
                // Start rendering sprites
                spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
                // Draw the game components (sprites included)
                base.Draw(gameTime);
                // End rendering sprites
                spriteBatch.End();

            }


            if (vplayer.State != MediaState.Stopped)
                videoTexture = vplayer.GetTexture();//Gets a texture from the video

            // Drawing to the rectangle will stretch the 
            // video to fill the screen
            Rectangle screen = new Rectangle(GraphicsDevice.Viewport.X,
                GraphicsDevice.Viewport.Y,
                GraphicsDevice.Viewport.Width,
                GraphicsDevice.Viewport.Height );
            
            // Draw the video, if we have a texture to draw.
            if (videoTexture != null)
            {
                spriteBatch.Begin();


                //spriteBatch.Draw(videoTexture, (new Vector2(0, 0)), screen,Color.White, 0, (new Vector2(0, 0)), 1, SpriteEffects.None, 0f); //draws video texture
                spriteBatch.Draw(videoTexture, screen, screen, Color.White, 0f, new Vector2(0, 0), SpriteEffects.None, 0f);
                spriteBatch.End();
            }

            // draw score, but only during actual game play
            spriteBatch.Begin();
#if XBOX360
            if (currentGameState == GameState.InGame)
            {
            spriteBatch.DrawString(gameFont, "Score: " + (bombCount+totalScore).ToString(), new Vector2(40, 30), Color.YellowGreen);
            }
#else
            if (currentGameState == GameState.InGame)
            {
                spriteBatch.DrawString(gameFont, "Score: " + (bombCount+totalScore).ToString(), new Vector2(15, 15), Color.YellowGreen);
            }
#endif
            spriteBatch.End();
        }
    }
}
